/*
 *    Copyright 2008 Intel Corporation
 * 
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 * 
 *        http://www.apache.org/licenses/LICENSE-2.0
 * 
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */

#ifndef _BPQ_BLOCK_H_
#define _BPQ_BLOCK_H_

#ifdef HAVE_CONFIG_H
#  include <dtn-config.h>
#endif

#include "BlockInfo.h"
namespace dtn {

class BPQFragment{
public:
	BPQFragment(size_t offset , size_t length) :
		offset_(offset),
		length_(length) {}

    ~BPQFragment() {}

    /// @{ Accessors
    size_t offset() const { return offset_; }
    size_t length() const { return length_; }
    /// @}

private:
    size_t offset_;              ///< Fragment offset
    size_t length_;              ///< Fragment length
};

class BPQBlock 
{
public:
    BPQBlock(Bundle* bundle);
    BPQBlock(BlockInfo* block);
    ~BPQBlock();

    int write_to_buffer(u_char* buf, size_t len);

    typedef enum {
        KIND_QUERY          = 0x00,
        KIND_RESPONSE       = 0x01,
    } kind_t;

    /// @{ Accessors
    kind_t          kind()          const { return kind_; }
    u_int           matching_rule() const { return matching_rule_; }
    u_int           query_len()     const { return query_len_; }
    u_char*         query_val()     const { return query_val_; }   
    u_int           length()        const;
    u_int			frag_len()		const { return fragments_.size(); }
    /// @}

    bool    match(const BPQBlock* other)  const;
    void add_fragment(BPQFragment fragment) {fragments_.push_back(fragment);}

    /// @{ Typedefs and wrappers for the BPQFragment vector and iterators
    typedef std::vector<BPQFragment> BPQFragmentVec;
    typedef BPQFragmentVec::iterator iterator;
    typedef BPQFragmentVec::const_iterator const_iterator;

    bool                           empty() const { return fragments_.empty(); }
    BPQFragmentVec::iterator       begin()       { return fragments_.begin(); }
    BPQFragmentVec::iterator       end()         { return fragments_.end();   }
    BPQFragmentVec::const_iterator begin() const { return fragments_.begin(); }
    BPQFragmentVec::const_iterator end()   const { return fragments_.end();   }
    /// @}

private:
    int initialise(BlockInfo* block);       ///< Wrapper function called by constructors

    kind_t kind_;               ///< Query || Response 
    u_int matching_rule_;       ///< Exact
    u_int query_len_;           ///< Length of the query value
    u_char* query_val_;         ///< Query value
    BPQFragmentVec fragments_;  ///< List of fragments returned
};

} // namespace dtn

#endif /* _BPQ_BLOCK_H_ */

